// jshint esversion: 6
	import * as THREE from 'three';
	import {FontLoader} from 'three/addons/loaders/FontLoader.js';
	import {TextGeometry} from 'three/addons/geometries/TextGeometry.js';
	

	// base KXS
	export function klystronx(text='K4', myscale=0.625) {
		const klystron = new THREE.Object3D();
		let group, textMesh, textGeo, materials;
		let size = 100;
		let textheight = 10;
		let pos = [];
		let rot = [];
		const hover = 30;
		const curveSegments = 8;
		let font;
		function insertText(isize, ipos, irot) {
			group = new THREE.Group();
			group.position.y = 100;
			klystron.add( group );
			size = isize;
			pos = ipos;
			rot = irot;
			loadFont();
		}
		function loadFont() {
			const loader = new FontLoader();
			// https://threejs.org/examples/fonts/droid_sans_bold.typeface.json
			loader.load( './fonts/droid_sans_bold.typeface.json', function ( response ) {
				font = response;
				refreshText();
			} );
		}
		materials = [
			new THREE.MeshPhongMaterial( { color: 0x333333, flatShading: true } ), // front
			new THREE.MeshPhongMaterial( { color: 0x333333 } ) // side
		];
		function createText() {
			textGeo = new TextGeometry(text, {font: font, size: size, height: textheight, curveSegments: curveSegments});
			textGeo.computeBoundingBox();
			const centerOffset = - 0.5 * ( textGeo.boundingBox.max.x - textGeo.boundingBox.min.x );
			textMesh = new THREE.Mesh( textGeo, materials );
			textMesh.position.x = pos[0];
			textMesh.position.y = pos[1];
			textMesh.position.z = pos[2];
			textMesh.rotation.x = rot[0];
			textMesh.rotation.y = rot[1];
			textMesh.rotation.z = rot[2];
			group.add( textMesh );
		}
		function refreshText() {
			group.remove( textMesh );
			if ( ! text ) return;
			createText();
		}
		insertText(100, [760,600,840], [0, Math.PI / 2, 0]);

		const width=1200, height=1200, depth=1200;
		const ModK = new THREE.Object3D();
		const ModContainer = new THREE.Object3D();
		const materialBase = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const materialGreyDark= new THREE.MeshLambertMaterial({color: 0x9d9d9d});
		const basegeometry = new THREE.BoxGeometry(width, height+200, depth);
		const d1mesh = new THREE.Mesh(basegeometry, materialBase);
		d1mesh.position.set(0, 100, -900);	
		ModContainer.add(d1mesh);

		// redcube
		const d2geometry = new THREE.BoxGeometry(width-400, height, depth);
		const dmaterial2= new THREE.MeshLambertMaterial({color: 0xe02900});
		const d2mesh = new THREE.Mesh(d2geometry, dmaterial2);
		d2mesh.position.set(200, 1400, -900);		
		ModContainer.add(d2mesh);	
		
  		// base
		const d3geometry = new THREE.BoxGeometry(width, height, depth);
		const d3mesh = new THREE.Mesh(d3geometry, materialBase);
		d3mesh.position.set(0, 0, 300);		
		ModContainer.add(d3mesh);	
	
		// big red cylinder
		const d4geometry = new THREE.CylinderGeometry(450, 450, 1200, 50);
		const material = new THREE.MeshLambertMaterial({color: 0xfc0303});	
		const d4mesh = new THREE.Mesh(d4geometry, material);
		d4mesh.rotateX(Math.PI * 1);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, 800 , 300);			
		ModContainer.add(d4mesh);	

		// yellow 1 cylinder
		const d5geometry = new THREE.CylinderGeometry(300, 370, 200, 50);
		const material2 = new THREE.MeshLambertMaterial({color: 0xecfc03});			
		const d5mesh = new THREE.Mesh(d5geometry, material2);
		d5mesh.rotateX(Math.PI * 1);
		d5mesh.rotateY(Math.PI * 1);
		d5mesh.rotateZ(Math.PI * 1);
		d5mesh.position.set(0, 1450 , 300);			
		ModContainer.add(d5mesh);	
		
		//  yellow 2 cylinder
		const d6geometry = new THREE.CylinderGeometry(150, 150, 250, 50);
		const material3= new THREE.MeshLambertMaterial({color: 0xecfc03});
		const d6mesh = new THREE.Mesh(d6geometry, material3);
		d6mesh.rotateX(Math.PI * 1);
		d6mesh.rotateY(Math.PI * 1);
		d6mesh.rotateZ(Math.PI * 1);
		d6mesh.position.set(0, 1680 , 300);			
		ModContainer.add(d6mesh);	

		//  yellow 3 cylinder
		const d7geometry = new THREE.CylinderGeometry(200, 200, 100, 50);
		const d7mesh = new THREE.Mesh(d7geometry, material3);
		d7mesh.rotateX(Math.PI * 1);
		d7mesh.rotateY(Math.PI * 1);
		d7mesh.rotateZ(Math.PI * 1);
		d7mesh.position.set(0, 1780 , 300);			
		ModContainer.add(d7mesh);			
		
		// tag
		const targhetta = new THREE.BoxGeometry( 10, 200, 300 );
		const materialwhite= new THREE.MeshLambertMaterial({color: 0xeffffff});
		const d14mesh = new THREE.Mesh(targhetta, materialwhite);
		d14mesh.position.set(-610, 600 , -600);
		ModContainer.add(d14mesh);	
		
		// torus red1 
		const torus_bs = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d15mesh = new THREE.Mesh( torus_bs, materialGreyDark );
		d15mesh.rotateX(Math.PI * 1);
		d15mesh.rotateY(Math.PI * 1.5);
		d15mesh.rotateZ(Math.PI * 1);	
		d15mesh.position.set(550, 2050 , -1400);
		ModContainer.add(d15mesh);	
	
		// torus red2
		const torus_bd = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d16mesh = new THREE.Mesh( torus_bd, materialGreyDark );
		d16mesh.rotateX(Math.PI * 1);
		d16mesh.rotateY(Math.PI * 1.5);
		d16mesh.rotateZ(Math.PI * 1);	
		d16mesh.position.set(-150, 2050 , -400);
		ModContainer.add(d16mesh);	
	
		// torus 1 white
		const torus_as = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d17mesh = new THREE.Mesh( torus_as, materialGreyDark );
		d17mesh.rotateX(Math.PI * 1);
		d17mesh.rotateY(Math.PI * 1.5);
		d17mesh.rotateZ(Math.PI * 1);	
		d17mesh.position.set(-400, 850 , -500);
		ModContainer.add(d17mesh);	

		// torus 2 white
		const torus_ad = new THREE.TorusGeometry( 60, 15, 20, 20, 7); 
		const d18mesh = new THREE.Mesh( torus_ad, materialGreyDark );
		d18mesh.rotateX(Math.PI * 1);
		d18mesh.rotateY(Math.PI * 1.5);
		d18mesh.rotateZ(Math.PI * 1);	
		d18mesh.position.set(-400, 850 , -1300);
		ModContainer.add(d18mesh);
		
		// line1
		const line1 = new THREE.BoxGeometry(10, 10, 1150);
		const d19mesh = new THREE.Mesh(line1, dmaterial2);
		d19mesh.position.set(-610, 750, -900);		
		ModContainer.add(d19mesh);		
	
		// line2
		const line2 = new THREE.BoxGeometry(10, 10, 1150);
		const d20mesh = new THREE.Mesh(line2, dmaterial2);
		d20mesh.position.set(-610, 550, 300);		
		ModContainer.add(d20mesh);			
		
		//text
/*		const loader = new FontLoader (); 
		loader.load( 'fonts/helvetiker_regular.typeface.json', function ( font ) {
    	const textGeometry = new TextGeometry('KX', { 
		font : font,
		size: 100,
		height: 100
   		});
		});	
	
 		const textMaterial = new THREE.MeshBasicMaterial({ color: 0xffffff });
		const textMesh = new THREE.Mesh(textGeometry, textMaterial);
		textMesh.position.set(-820, 600, -600);
    	ModContainer.add(textMesh);		
*/	
		ModContainer.rotateY(Math.PI);
		ModContainer.scale.set(myscale, myscale, myscale);
		ModContainer.position.set(width*myscale/2, height*myscale/2, depth*myscale/2);
		klystron.add(ModContainer);
		return klystron;
	}
